@@ -15,6 +15,7 @@
 #undef HAVE_SETENV
 #undef HAVE_UNSETENV
 #undef HAVE_CLEARENV
+#undef HAVE_SETRESUID
 #undef EXPANDED_DATADIR
 #undef EXPANDED_PIXMAPDIR
 #undef EXPANDED_BINDIR
@@ -25,3 +26,4 @@
 #undef EXPANDED_SYSCONFDIR
 #undef EXPANDED_SESSDIR
 #undef GETTEXT_PACKAGE
+#undef HAVE_LOGINCAP
