@@ -13,6 +13,7 @@
 
 #include "gdm.h"
 #include "gdmwm.h"
+#include "vicious.h"
 
 #include "greeter.h"
 #include "greeter_configuration.h"
@@ -21,6 +22,7 @@
 #include "greeter_item_clock.h"
 #include "greeter_item_pam.h"
 #include "greeter_item_capslock.h"
+#include "greeter_item_timed.h"
 #include "greeter_events.h"
 #include "greeter_action_language.h"
 #include "greeter_session.h"
@@ -47,10 +49,14 @@ gchar *GdmSuspend = NULL;
 gchar *GdmConfigurator = NULL;
 gboolean GdmSystemMenu = TRUE;
 gboolean GdmConfigAvailable = TRUE;
+gboolean GdmTimedLoginEnable;
+gchar *GdmTimedLogin;
+gint GdmTimedLoginDelay;
 
 gboolean GdmUseCirclesInEntry = FALSE;
 
 static gboolean used_defaults = FALSE;
+gint greeter_current_delay = 0;
 
 extern gboolean session_dir_whacked_out;
 
@@ -58,10 +64,11 @@ extern gboolean session_dir_whacked_out;
 static void 
 greeter_parse_config (void)
 {
-    if (!g_file_test (GDM_CONFIG_FILE, G_FILE_TEST_EXISTS)) {
+    if (!g_file_test (GDM_CONFIG_FILE, G_FILE_TEST_EXISTS))
+      {
 	syslog (LOG_ERR, _("greeter_parse_config: No configuration file: %s. Using defaults."), GDM_CONFIG_FILE);
 	used_defaults = TRUE;
-    }
+      }
 
     gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 
@@ -96,11 +103,41 @@ greeter_parse_config (void)
     GdmReboot = gnome_config_get_string (GDM_KEY_REBOOT);
     GdmSuspend = gnome_config_get_string (GDM_KEY_SUSPEND);
     GdmConfigurator = gnome_config_get_string (GDM_KEY_CONFIGURATOR);
+
+    GdmTimedLoginEnable = gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE);
+
+    /* Note: TimedLogin here is not gotten out of the config
+     * but from the daemon since it's been munged on by the daemon a bit
+     * already maybe */
+    if (GdmTimedLoginEnable)
+      {
+        GdmTimedLogin = g_strdup (g_getenv("GDM_TIMED_LOGIN_OK"));
+	if (ve_string_empty (GdmTimedLogin))
+	  {
+	    g_free (GdmTimedLogin);
+	    GdmTimedLogin = NULL;
+	  }
+
+	GdmTimedLoginDelay =
+		gnome_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+	if (GdmTimedLoginDelay < 5)
+	  {
+	    syslog (LOG_WARNING,
+		    _("TimedLoginDelay was less then 5.  I'll just use 5."));
+	    GdmTimedLoginDelay = 5;
+	  }
+      }
+    else
+      {
+        GdmTimedLogin = NULL;
+	GdmTimedLoginDelay = 5;
+      }
+    greeter_current_delay = GdmTimedLoginDelay;
     
     gnome_config_pop_prefix();
 
     if (GdmXineramaScreen < 0)
-	    GdmXineramaScreen = 0;
+      GdmXineramaScreen = 0;
 }
 
 static gboolean
@@ -115,6 +152,7 @@ greeter_ctrl_handler (GIOChannel *source,
     char *session;
     GreeterItemInfo *conversation_info;
     GreeterItemInfo *entry_info;
+    static GnomeCanvasItem *disabled_cover = NULL;
     gchar *language;
 
     /* If this is not incoming i/o then return */
@@ -145,13 +183,14 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 	
 	greeter_item_pam_set_user (buf);
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
     case GDM_LOGIN:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	greeter_item_pam_prompt (tmp, 32, TRUE, TRUE);
 	g_free (tmp);
 	break;
@@ -160,7 +199,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	greeter_item_pam_prompt (tmp, 128, TRUE, FALSE);
 	g_free (tmp);
 	break;
@@ -169,7 +208,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	greeter_item_pam_prompt (tmp, 128, FALSE, FALSE);
 	g_free (tmp);
 	break;
@@ -177,10 +216,11 @@ greeter_ctrl_handler (GIOChannel *source,
     case GDM_MSG:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	greeter_item_pam_message (tmp);
 	g_free (tmp);
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 
 	break;
 
@@ -188,11 +228,12 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	greeter_item_pam_error (tmp);
 	g_free (tmp);
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_ERRDLG:
@@ -202,7 +243,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	/* we should be now fine for focusing new windows */
 	gdm_wm_focus_new_windows (TRUE);
 
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	dlg = gtk_message_dialog_new (NULL /* parent */,
 				      GTK_DIALOG_MODAL /* flags */,
 				      GTK_MESSAGE_ERROR,
@@ -218,18 +259,20 @@ greeter_ctrl_handler (GIOChannel *source,
 	gtk_widget_destroy (dlg);
 	gdm_wm_no_login_focus_pop ();
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_SESS:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 	buf[len-1] = '\0';
 
-	tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	tmp = ve_locale_to_utf8 (buf);
 	session = greeter_session_lookup (tmp);
 	g_free (tmp);
-	tmp = g_locale_from_utf8 (session, -1, NULL, NULL, NULL);
-	g_print ("%c%s\n", STX, tmp);
+	tmp = ve_locale_from_utf8 (session);
+	printf ("%c%s\n", STX, tmp);
+	fflush (stdout);
 	g_free (session);
 	g_free (tmp);
 	break;
@@ -238,7 +281,8 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 	buf[len-1] = '\0';
 	language = greeter_language_get_language (buf);
-	g_print ("%c%s\n", STX, language);
+	printf ("%c%s\n", STX, language);
+	fflush (stdout);
 	g_free (language);
 	break;
 
@@ -246,9 +290,10 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
 	if (greeter_save_session ())
-	  g_print ("%cY\n", STX);
+	  printf ("%cY\n", STX);
 	else
-	  g_print ("%c\n", STX);
+	  printf ("%c\n", STX);
+	fflush (stdout);
 	
 	break;
 
@@ -256,9 +301,10 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
 	if (greeter_language_get_save_language ())
-	    g_print ("%cY\n", STX);
+	    printf ("%cY\n", STX);
 	else
-	    g_print ("%c\n", STX);
+	    printf ("%c\n", STX);
+	fflush (stdout);
 
 	break;
 
@@ -273,24 +319,27 @@ greeter_ctrl_handler (GIOChannel *source,
 	
 	if (conversation_info)
 	  {
-	    tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	    tmp = ve_locale_to_utf8 (buf);
 	    g_object_set (G_OBJECT (conversation_info->item),
 			  "text", tmp,
 			  NULL);
 	    g_free (tmp);
 	  }
 
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_QUIT:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
-	/* TODO: Stuff. */
+	greeter_item_timed_stop ();
+	greeter_item_pam_leftover_messages ();
 
 	gdk_flush ();
 
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 
 	/* screw gtk_main_quit, we want to make sure we definately die */
 	_exit (EXIT_SUCCESS);
@@ -304,7 +353,7 @@ greeter_ctrl_handler (GIOChannel *source,
 		do {
 			g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 			buf[len-1] = '\0';
-			tmp = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+			tmp = ve_locale_to_utf8 (buf);
 			g_string_append (str, tmp);
 			g_free (tmp);
 		} while (len == PIPE_SIZE-1);
@@ -314,8 +363,9 @@ greeter_ctrl_handler (GIOChannel *source,
 
 		g_string_free (str, TRUE);
 
-		tmp = g_locale_from_utf8 (sess, -1, NULL, NULL, NULL);
-		g_print ("%c%s\n", STX, tmp);
+		tmp = ve_locale_from_utf8 (sess);
+		printf ("%c%s\n", STX, tmp);
+		fflush (stdout);
 		g_free (tmp);
 
 		g_free (sess);
@@ -326,46 +376,64 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
 	if (greeter_save_gnome_session ())
-	    g_print ("%cY\n", STX);
+	    printf ("%cY\n", STX);
 	else
-	    g_print ("%c\n", STX);
+	    printf ("%c\n", STX);
+	fflush (stdout);
 
 	break;
 
     case GDM_STARTTIMER:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
-	/*
-	 * Timed Login: Start Timer Loop
-	 */
-
-	/* TODO */
+	greeter_item_timed_start ();
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_STOPTIMER:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
 
-	/*
-	 * Timed Login: Stop Timer Loop
-	 */
+	greeter_item_timed_stop ();
 
-	/* TODO */
-	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_DISABLE:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
-	/* TODO */
-	g_print ("%c\n", STX);
+	gtk_widget_set_sensitive (window, FALSE);
+
+	if (disabled_cover == NULL)
+	  {
+	    disabled_cover = gnome_canvas_item_new
+		    (gnome_canvas_root (GNOME_CANVAS (canvas)),
+		     GNOME_TYPE_CANVAS_RECT,
+		     "x1", 0.0,
+		     "y1", 0.0,
+		     "x2", (double)canvas->allocation.width,
+		     "y2", (double)canvas->allocation.height,
+		     "fill_color_rgba", (guint)0x00000088,
+		     NULL);
+	  }
+
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_ENABLE:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
-	/* TODO */
-	g_print ("%c\n", STX);
+	gtk_widget_set_sensitive (window, TRUE);
+
+	if (disabled_cover != NULL)
+	  {
+	    gtk_object_destroy (GTK_OBJECT (disabled_cover));
+	    disabled_cover = NULL;
+	  }
+
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     /* These are handled separately so ignore them here and send
@@ -373,7 +441,8 @@ greeter_ctrl_handler (GIOChannel *source,
     case GDM_NEEDPIC:
     case GDM_READPIC:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_NOFOCUS:
@@ -381,7 +450,8 @@ greeter_ctrl_handler (GIOChannel *source,
 
 	gdm_wm_no_login_focus_push ();
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 
     case GDM_FOCUS:
@@ -389,7 +459,8 @@ greeter_ctrl_handler (GIOChannel *source,
 
 	gdm_wm_no_login_focus_pop ();
 	
-	g_print ("%c\n", STX);
+	printf ("%c\n", STX);
+	fflush (stdout);
 	break;
 	
     default:
@@ -419,6 +490,7 @@ greeter_setup_items (void)
   greeter_item_clock_setup ();
   greeter_item_pam_setup ();
   greeter_item_capslock_setup (window);
+  greeter_item_timed_setup ();
   greeter_item_register_action_callback ("language_button",
 					 greeter_action_language,
 					 window);
@@ -610,6 +682,8 @@ greeter_query (const gchar *msg)
 				      GTK_BUTTONS_YES_NO,
 				      "%s",
 				      msg);
+	gtk_label_set_use_markup
+		(GTK_LABEL (GTK_MESSAGE_DIALOG (req)->label), TRUE);
 
 	g_signal_connect (G_OBJECT (req), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
@@ -774,7 +848,7 @@ main (int argc, char *argv[])
     setlocale (LC_ALL, GdmDefaultLocale);
   else
     setlocale (LC_ALL, "");
-  
+
   gtk_init (&argc, &argv);
 
   setup_cursor (GDK_LEFT_PTR);
